package com.sd.business.service.area.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.area.bo.SetAreaName;
import com.sd.business.entity.area.dto.AreaSelectDto;
import com.sd.business.entity.area.po.Area;
import com.sd.business.entity.area.vo.AreaVo;
import com.sd.business.mapper.area.AreaMapper;
import com.sd.business.service.area.AreaService;
import com.sd.framework.util.sql.Sql;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * <p>
 * 中国行政地区表 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-20
 */
@Service
public class AreaServiceImpl extends ServiceImpl<AreaMapper, Area> implements AreaService {

    @Override
    public List<AreaVo> getList(AreaSelectDto dto) {

        return Sql.create(AreaVo.class)
                .select(Area::getAreaCode, Area::getName)
                .from(Area.class)
                .eq(Area::getParentCode, dto.getParentCode())
                .list();
    }

    @Override
    public void setAreaName(List<? extends SetAreaName> list) {

        List<Long> codeList = list.stream().flatMap(item -> {
            List<Long> itemList = new ArrayList<>();
            itemList.add(item.getProvinceCode());
            itemList.add(item.getCityCode());
            itemList.add(item.getCountyCode());
            itemList.add(item.getStreetCode());
            return itemList.stream();
        }).filter(Objects::nonNull).distinct().collect(Collectors.toList());

        if (codeList.isEmpty()) {
            return;
        }

        Map<Long, String> map = mapKV(Area::getAreaCode, Area::getName, q -> q.in(Area::getAreaCode, codeList));

        for (SetAreaName item : list) {
            item.setProvinceName(map.get(item.getProvinceCode()));
            item.setCityName(map.get(item.getCityCode()));
            item.setCountyName(map.get(item.getCountyCode()));
            item.setStreetName(map.get(item.getStreetCode()));
        }

    }

}
