package com.sd.business.service.bom.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sd.business.entity.bom.dto.BomSpecDto;
import com.sd.business.entity.bom.dto.BomSpecSelectDto;
import com.sd.business.entity.bom.po.Bom;
import com.sd.business.entity.bom.po.BomClassify;
import com.sd.business.entity.bom.po.BomSpec;
import com.sd.business.entity.bom.vo.BomSpecVo;
import com.sd.business.mapper.bom.BomSpecMapper;
import com.sd.business.service.bom.BomSpecService;
import com.sd.framework.util.Assert;
import com.sd.framework.util.sql.Sql;
import org.springframework.stereotype.Service;


/**
 * <p>
 * bom规格 服务实现类
 * </p>
 *
 * @author
 * @since 2023-11-20
 */
@Service
public class BomSpecServiceImpl extends ServiceImpl<BomSpecMapper, BomSpec> implements BomSpecService {

    @Override
    public Page<BomSpecVo> getPage(BomSpecSelectDto dto) {
        Page<BomSpecVo> page = Sql.create(BomSpecVo.class)
                .selectAll(BomSpec.class)
                .select(Bom::getFrontGrain)
                .select(Bom::getReverseGrain)
                .select(Bom::getChromatophore)
                .from(BomSpec.class)
                .innerJoin(Bom.class, Bom::getId, BomSpec::getBomId)
                .innerJoin(BomClassify.class, BomClassify::getId, Bom::getBomClassifyId)
                .eq(BomClassify::getCode, dto.getBomClassifyCode())
                .eq(BomSpec::getLength, dto.getLength())
                .eq(BomSpec::getWidth, dto.getWidth())
                .eq(BomSpec::getHeight, dto.getHeight())
                .eq(BomSpec::getColour, dto.getColour())
                .eq(Bom::getChromatophore, dto.getChromatophore())
                .eq(Bom::getFrontGrain, dto.getFrontGrain())
                .eq(Bom::getReverseGrain, dto.getReverseGrain())
                .orderByDesc(BomSpec::getId)
                .page(dto);

        return page;
    }

    @Override
    public BomSpecVo detail(Long id) {

        BomSpecVo vo = Sql.create(BomSpecVo.class)
                .selectAll(BomSpec.class)
                .select(Bom::getFrontGrain)
                .select(Bom::getReverseGrain)
                .select(Bom::getChromatophore)
                .select(Bom::getSpecies)
                .from(BomSpec.class)
                .innerJoin(Bom.class, Bom::getId, BomSpec::getBomId)
                .eq(BomSpec::getId, id)
                .one();

        Assert.notNull(vo, "未知数据");

        return vo;
    }

    @Override
    public void add(BomSpecDto dto) {
        save(dto);
    }

    @Override
    public void edit(BomSpecDto dto) {
        updateById(dto);
    }

    @Override
    public void delete(Long id) {
        removeById(id);
    }

}
